/****************************************************************************************
*                               TERMS OF USE: MIT License                               *
*                                                                                       *
* Permission is hereby granted, free of charge, to any person obtaining a copy of this  *
* software and associated documentation files (the "Software"), to deal in the Software *
* without restriction, including without limitation the rights to use, copy, modify,    *
* merge, publish, distribute, sublicense, and/or sell copies of the Software, and to    *
* permit persons to whom the Software is furnished to do so, subject to the following   *
* conditions:                                                                           *
*                                                                                       *
* The above copyright notice and this permission notice shall be included in all copies *
* or substantial portions of the Software.                                              *
*                                                                                       *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,   *
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A         *
* PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT    *
* HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF  *
* CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE  *
* OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                         *
****************************************************************************************/

#ifndef ffds1_Class_Defined__
#define ffds1_Class_Defined__

#include <stdint.h>

#define Buf_len (987)
#define Min_half_period (86)
#define Xon (17)
#define Xoff (19)

typedef struct ffds1 {
  char volatile * Write_buf_ptr;
  volatile char Send_temp;
  int32_t Half_bit_period;
  volatile uint16_t Rx_head, Rx_tail;
  volatile uint8_t Rx_buffer[Buf_len];
  uint8_t Cog;
} ffds1;


  /** Starts the ffds1 PSAM engine running in a new cog
   * rx_pin: The pin number (0-31) for incoming data
   * tx_pin: The pin number (0-31) for outgoing data
   *        Must not be the same as rx_pin. Use -1 to disable TX
   * rate : Initial baud rate
   *
   * Returns TRUE if the cog started, FALSE if not.
   **/
  int ffds1_start(int Rx_pin, int Tx_pin, int Rate);

  /** Stops the serial engine
   **/
  void ffds1_stop(void);

  /** Performs a live update of the Baud rate in the serial engine
   * rate : Desired baudrate of the serial engine
   *
   * returns TRUE if the baud rate was achieved.
   **/
  int ffds1_setbaud(int Rate);

  /** Performs a live update of the Baud rate in the serial engine
   * rate : Desired baudrate of the serial engine
   * sysclock : System clock; use CLKFREQ, or provide your own
   *
   * returns TRUE if the baud rate was achieved.
   **/
  int ffds1_setbaudclock(int Rate, int Sysclock);

  /** Sends a string over the serial line.
   * buf: a pointer to a NULL-terminated string in Hub RAM
   **/
  void ffds1_str(char *buf);

  /** Sends down a single character.
   **/
  void fds1_tx(char Char_val);

  /** Sends a raw buffer over the serial link, waiting until it's complete.
   * Buf: The buffer to send
   * Buffer_bytes: How many bytes to send.
   **/
  void ffds1_txbuf(char *buf, size_t count);

  /** Sends a raw buffer over the serial link, but returns immediately.
   * Buf: The buffer to send
   * Buffer_bytes: How many bytes to send.
   **/
  void ffds1_txbufnowait(char *buf, size_t count);

  /** Blocking wait until transmit cog is finished with hub RAM.
   * Only usefulf or txBufNoWait
   **/
  void ffds1_waitfortx();

  /** Flushes receive buffer
   **/
  void ffds1_rxflush();

  /** Checks if there's a byte in the receive buffer
   * Returns: -1 if no char, otherwise next char in buffer
   **/
  int ffds1_rxcheck();

  /** Waits for a byte to be received or a timeout to occur.
   * Ms: Timeout in MSec
   * Returns: -1 if timeout, otherwise next char in buffer
   **/
  int ffds1_rxtime(int Ms);

  /** Receives a byte, and will wait forever to get one.
   **/
  char ffds1_rx(void);

  /** Sends a decimal number over the serial link.
   * Value: The number to send
   **/
  void ffds1_dec(int Value);

  /** Sends a hexadecimal number over the serial link.
   * Value: The number to send
   * Digits: How many hex digits to send. (typically 2 per byte)
   **/
  void ffds1_hex(int Value, int Digits);

  /** Sends a binary number over the serial link.
   * Value: The number to send
   * Digits: How many bits to send. (typically 8 per byte)
   **/
  void ffds1_bin(int Value, int Digits);
#endif
